from radiant.framework.server import RadiantAPI, RadiantServer, render
from .figurestream import FigureStream


########################################################################
class FrameworkAPI(RadiantAPI):
    """"""


FrameworkServer = None




import asyncio

from dunderlab.api import aioAPI as API
from chaski.streamer import ChaskiStreamer

TOKEN = {
    "refresh": "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ0b2tlbl90eXBlIjoicmVmcmVzaCIsImV4cCI6MTcyNTIyODA1OSwiaWF0IjoxNjkzNjkyMDU5LCJqdGkiOiIyMjg3Zjc2MTk5MDY0MzJiYTc0ZTZiMWQ4NWNiYzBiNCIsInVzZXJfaWQiOjF9.6wq-irScHTJ8nf8U0ruByrrYiEvEkqAbZFKTyF_A8Yc",
    "access": "eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.eyJ0b2tlbl90eXBlIjoiYWNjZXNzIiwiZXhwIjoxNjkzNzc4NDU5LCJpYXQiOjE2OTM2OTIwNTksImp0aSI6ImRhZDgzNTU3NTk3MDQ3YzA5NDZhZWQ1OTczZmU5OWNkIiwidXNlcl9pZCI6MX0.RcO7-ms8tj8F-9JkOzfACYMhtByi7BKspOla7bhpAK0"
}

api = API('http://timescaledb-api-worker/timescaledbapp/', token=TOKEN['access'])


# ----------------------------------------------------------------------
def write_on_api(data):
    """"""
    global api

    endpoint = data['endpoint']
    params = data['params']
    await getattr(api, endpoint).post(params)


# ----------------------------------------------------------------------
async def run():
    """"""
    consumer = ChaskiStreamer(
        ip='0.0.0.0',
        port=51113,
        name='Foundation Chaski TimeScaleDB-API',
        run=False,
        subscriptions=['timescaledb-api'],
    )

    await consumer.run()
    async with consumer as message_queue:
        async for incoming_message in message_queue:
            write_on_api(incoming_message.data)


asyncio.run(run())



"""
Django settings for timescaledb_api project.

Generated by 'django-admin startproject' using Django 4.2.3.

For more information on this file, see
https://docs.djangoproject.com/en/4.2/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/4.2/ref/settings/
"""

from datetime import timedelta
from pathlib import Path

# Build paths inside the project like this: BASE_DIR / 'subdir'.
BASE_DIR = Path(__file__).resolve().parent.parent


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/4.2/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "django-insecure-pp5mz*5%1r6k8wp2ln#i-&jfkso61cp45syb80j$5r7cwq(2&3"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ['*']


# Application definition

INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    'django_extensions',
    'django_filters',
    'rest_framework',
    'rest_framework_simplejwt',
    'dunderlab.django.timescaledbapp.apps.TimeScaleDBConfig',
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "whitenoise.middleware.WhiteNoiseMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    # "django.middleware.gzip.GZipMiddleware",
]

ROOT_URLCONF = "timescaledb_api.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "timescaledb_api.wsgi.application"


# Database
# https://docs.djangoproject.com/en/4.2/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": BASE_DIR / "db.sqlite3",
    },

    'timescaledb': {
        'ENGINE': 'django.db.backends.postgresql',
        'NAME': 'timescaledb',
        'USER': 'postgres',
        'PASSWORD': 'password',
        'HOST': 'timescaledb-service',
        'PORT': '5432',
    },
}

DATABASE_ROUTERS = [
    'dunderlab.django.timescaledbapp.db_router.TimeScaleDBRouter',
]

# Password validation
# https://docs.djangoproject.com/en/4.2/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]


# Internationalization
# https://docs.djangoproject.com/en/4.2/topics/i18n/

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/4.2/howto/static-files/

STATIC_URL = "static/"
STATIC_ROOT = BASE_DIR.joinpath('resources')

# Default primary key field type
# https://docs.djangoproject.com/en/4.2/ref/settings/#default-auto-field

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

APPEND_SLASH = True

STATICFILES_DIRS = [
    BASE_DIR.joinpath('static'),
]

STATICFILES_FINDERS = [
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
]

REST_FRAMEWORK = {

    'DEFAULT_AUTHENTICATION_CLASSES': [
        'rest_framework_simplejwt.authentication.JWTAuthentication',
        # 'rest_framework_simplejwt.authentication.JWTStatelessUserAuthentication',

        # To use the DRF Web browsable API
        'rest_framework.authentication.SessionAuthentication',
        'rest_framework.authentication.BasicAuthentication',
    ],

    # Use Django's standard `django.contrib.auth` permissions,
    # or allow read-only access for unauthenticated users.
    'DEFAULT_PERMISSION_CLASSES': [
        'rest_framework.permissions.IsAuthenticated',
        # 'rest_framework.permissions.AllowAny',
    ],
    'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.PageNumberPagination',
    # 'DEFAULT_PAGINATION_CLASS': 'rest_framework.pagination.CursorPagination',
    'PAGE_SIZE': 16,

    'DEFAULT_FILTER_BACKENDS': ['django_filters.rest_framework.DjangoFilterBackend'],
}


STATICFILES_STORAGE = "whitenoise.storage.CompressedManifestStaticFilesStorage"

SIMPLE_JWT = {
    'ACCESS_TOKEN_LIFETIME': timedelta(days=1),
    'REFRESH_TOKEN_LIFETIME': timedelta(days=365),
    # 'ROTATE_REFRESH_TOKENS': False,
    # 'BLACKLIST_AFTER_ROTATION': True,
    # 'UPDATE_LAST_LOGIN': False,

    # 'ALGORITHM': 'HS256',
    # 'SIGNING_KEY': SECRET_KEY,
    # 'VERIFYING_KEY': None,
    # 'AUDIENCE': None,
    # 'ISSUER': None,

    # 'AUTH_HEADER_TYPES': ('Bearer',),
    # 'USER_ID_FIELD': 'id',
    # 'USER_ID_CLAIM': 'user_id',

    # 'AUTH_TOKEN_CLASSES': ('rest_framework_simplejwt.tokens.AccessToken',),
    # 'TOKEN_TYPE_CLAIM': 'token_type',

    # 'SLIDING_TOKEN_REFRESH_EXP_CLAIM': 'refresh_exp',
    # 'SLIDING_TOKEN_LIFETIME': timedelta(minutes=5),
    # 'SLIDING_TOKEN_REFRESH_LIFETIME': timedelta(days=1),
}

TIMESCALEDB_CHUNK_INTERVAL = "1 hours"
TIMESCALEDB_RETENTION_INTERVAL = "60 seconds"
TIMESCALEDB_SCHEDULE_INTERVAL = "60 seconds"



from foundation.radiant.utils import environ
from confluent_kafka import Producer
import logging

level = logging.WARNING


########################################################################
class KafkaLogging(logging.Handler):
    """"""

    # ----------------------------------------------------------------------
    def __init__(self):
        """"""
        super().__init__()

        self.topic = environ('WORKER_NAME', 'logs')
        formatter = logging.Formatter(f'%(levelname)s: {self.topic} [%(asctime)s]: %(message)s')
        self.setFormatter(formatter)
        self.producer = Producer({'bootstrap.servers': 'kafka-logs-service:9093'})

    # ----------------------------------------------------------------------
    def emit(self, record):
        """"""
        log_message = self.format(record)
        self.producer.produce(self.topic, value=log_message)
        self.producer.flush()

    # ----------------------------------------------------------------------
    @property
    def kafka_available(self):
        """"""
        try:
            self.producer.list_topics(timeout=0.5)
            return True
        except:
            return False


try:
    custom_handler = KafkaLogging()
    if custom_handler.kafka_available:
        custom_handler.setLevel(level)
        root_logger = logging.getLogger()
        root_logger.setLevel(level)
        root_logger.addHandler(custom_handler)
except Exception as e:
    logging.warning('Impossible to connect logging with Kafka')
    logging.warning(str(e))



"""
WSGI config for timescaledb_api project.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/4.2/howto/deployment/wsgi/
"""

import os
import sys
from django.core.wsgi import get_wsgi_application

sys.path.append(os.path.abspath(os.path.dirname(os.path.dirname(__file__))))
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "timescaledb_api.settings")

application = get_wsgi_application()
